/**************************************************************************
  This is an example for our Monochrome OLEDs based on SSD1306 drivers

  Pick one up today in the adafruit shop!
  ------> http://www.adafruit.com/category/63_98

  This example is for a 128x64 pixel display using I2C to communicate
  3 pins are required to interface (two I2C and one reset).

  Adafruit invests time and resources providing this open
  source code, please support Adafruit and open-source
  hardware by purchasing products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries,
  with contributions from the open source community.
  BSD license, check license.txt for more information
  All text above, and the splash screen below must be
  included in any redistribution.

  debounce button code: https://www.arduino.cc/en/Tutorial/BuiltInExamples/Debounce
 **************************************************************************/
#include <HX711_ADC.h>
#include <EEPROM.h>

#include "SSD1306Ascii.h"
#include "SSD1306AsciiAvrI2c.h"

// 0X3C+SA0 - 0x3C or 0x3D
#define I2C_ADDRESS 0x3C

// Define proper RST_PIN if required.
#define RST_PIN -1

SSD1306AsciiAvrI2c oled;

// weight scale.
const int HX711_dout = 4; //mcu > HX711 dout pin
const int HX711_sck = 5; //mcu > HX711 sck pin
HX711_ADC LoadCell(HX711_dout, HX711_sck);
unsigned long t = 0; // timer for weight scale.
bool refreshScreen = true;

// pin of buttons
const int PIN_BUTTON_MOVE_UP = 2;
const int PIN_BUTTON_CLICK = 3;
const int PIN_BUTTON_MOVE_DOWN = 6;
const int PIN_WATERPUMP = 7;


// variables for debouncing the buttons
unsigned long lastDebounceTime_Click = 0;
unsigned long lastDebounceTime_mUp = 0;
unsigned long lastDebounceTime_mDown = 0;
const int debounceDelay = 30;

int lastButtonState_Click = LOW;
int lastButtonState_mUp = LOW;
int lastButtonState_mDown = LOW;

int buttonState_Click = LOW;
int buttonState_mUp = LOW;
int buttonState_mDown = LOW;

bool hasDoneAction_Click = false;
bool hasDoneAction_mUp = false;
bool hasDoneAction_mDown = false;

int pastPlantWeight[2];
unsigned long timerPlantWeight = 0;
int plantState = 0;

// for the switch case
int state = 1;
int menuStatus = 1;

int currentWeight;
int minWeightPlant = 100;
int weightWatering = 50;


int oldMinWeightPlant;
int oldWeightWatering;
int oldMenuStatus;
int oldState;
int oldCurrentWeight;

const int EEPROM_WEIGHT_PLANT = 1;
const int EEPROM_WATER_WEIGHT = 2;

void setup() {
  Serial.begin(9600);


  pinMode(PIN_BUTTON_CLICK, INPUT);
  pinMode(PIN_BUTTON_MOVE_DOWN, INPUT);
  pinMode(PIN_BUTTON_MOVE_UP, INPUT);
  pinMode(PIN_WATERPUMP, OUTPUT);

  // for OLED initialisation
#if RST_PIN >= 0
  oled.begin(&Adafruit128x64, I2C_ADDRESS, RST_PIN);
#else // RST_PIN >= 0
  oled.begin(&Adafruit128x64, I2C_ADDRESS);
#endif // RST_PIN >= 0
  oled.clear();

  // for 5g load cell initialisation
  LoadCell.begin();
  float calibrationValue = 467.23;
  unsigned long stabilizingtime = 2000; // preciscion right after power-up can be improved by adding a few seconds of stabilizing time
  boolean _tare = true;
  LoadCell.start(stabilizingtime, _tare);
  LoadCell.setCalFactor(calibrationValue);

  timerPlantWeight = millis(); // run timer for hourly check on plant weight

  //  retrieve EEPROM values
  minWeightPlant = EEPROM.read(EEPROM_WEIGHT_PLANT);
  weightWatering = EEPROM.read(EEPROM_WATER_WEIGHT);

  pastPlantWeight[0] = 300;
  pastPlantWeight[1] = 300;
  pastPlantWeight[2] = 300;
  Serial.println(minWeightPlant);
  Serial.println(weightWatering);
  Serial.println("done with setup");

}


void loop() {
   givePlantWaterNow(); // when you press the moveUp and moveDown button at the same time, water the plant now
  debouncingButtons();
  weighScale(); // weigh the plant function.
  refreshScreenNow(); // refresh menu screen
  weighPlant(); // check plant weight every hour, put data in array
 

 if ( buttonState_Click == HIGH && hasDoneAction_Click == false && buttonState_mDown == HIGH && hasDoneAction_mDown == false  ) {
    hasDoneAction_Click = true;
    hasDoneAction_mDown = true;

    digitalWrite(PIN_WATERPUMP, LOW);

  }

  switch (state) {
    case 1:
      if (hasDoneAction_Click == false && buttonState_Click == HIGH) {
        // when any button pressed, go to the menu (case 2)
        hasDoneAction_Click = true; // for debouncing
        Serial.println("go to menu");
        state = 2;

      }
      break;

    case 2:
      Menu2_SettingsLogic();
      if (menuStatus == 4) {
        // when selecting the fourth option in the menu, go back to case 1 and reset the menu status for menu 2
        menuStatus = 1;
        state = 1;
      }
      break;
  }

  if (checkPlantWeight() == true ) {
    // when in the past three hours the recorded weight of the plant was under the minimum weight, give plant water
    givePlantWater();
  }

}

void givePlantWaterNow() {
  if ( buttonState_mUp == HIGH && hasDoneAction_mUp == false && buttonState_mDown == HIGH && hasDoneAction_mDown == false ) {
    hasDoneAction_mUp = true;
    hasDoneAction_mDown = true;
    digitalWrite(PIN_WATERPUMP, HIGH);

  }
    if ( buttonState_Click == HIGH && hasDoneAction_Click == false ) {
    hasDoneAction_Click = true;

    digitalWrite(PIN_WATERPUMP, LOW);

  }
}


void givePlantWater() {
  if (currentWeight < minWeightPlant + weightWatering  ) {
    // when the current weight is less than the minimum weight plus the amount of water that's added when watering
    // OR when the move up, move down button get pressed at the same time
    digitalWrite(PIN_WATERPUMP, HIGH);
  } else {
    // fix the plant weight data, so it doesn't water automatically again
    if (plantState != 3) {
      plantState + 1;
    } else {
      plantState = 0;
    }
    pastPlantWeight[plantState] = currentWeight;

    digitalWrite(PIN_WATERPUMP, LOW);
  }
}

boolean checkPlantWeight() {
  if (pastPlantWeight[0] < minWeightPlant &&  pastPlantWeight[1] < minWeightPlant && pastPlantWeight[2] < minWeightPlant ) {
    return true;
  } else {
    return false;
  }
}

void weighPlant() {
  const int plantIntervalTimer = 3600000; // 1 hour between measuring weight

  if (millis() > plantIntervalTimer + timerPlantWeight) {
    timerPlantWeight = millis();
    pastPlantWeight[plantState] = currentWeight;
    plantState++;
    Serial.println(plantState);
    if (plantState == 3) {
      plantState = 0;
    }

  }
}

void refreshScreenNow() {
  // refresh screen ook wanneer: weight verandert bij de array.

  if ( oldState != state || oldMenuStatus != menuStatus || oldMinWeightPlant != minWeightPlant || oldWeightWatering != weightWatering
       || oldCurrentWeight != currentWeight) {
    refreshScreen = true;
  }

  if (refreshScreen == true) {
    oled.clear();
    if (state == 1) {
      Menu1_CurrentStats();
    }
    if (state == 2) {
      Menu2_Settings();
    }
    oldState = state;
    oldMenuStatus = menuStatus;
    oldMinWeightPlant = minWeightPlant;
    oldWeightWatering = weightWatering;
    oldCurrentWeight = currentWeight;
    refreshScreen = false;
  }
}


void weighScale() {
  static boolean newDataReady = 0;
  const int serialPrintInterval = 50; //increase value to slow down when it weighs
  // 36 000 000 for 1 hr


  // check for new data/start next conversion:
  if (LoadCell.update()) newDataReady = true;

  // get smoothed value from the dataset:
  if (newDataReady) {
    if (millis() > t + serialPrintInterval) {
      float i = LoadCell.getData();
      currentWeight = i;
      newDataReady = 0;
      t = millis();
    }
  }

}



void Menu1_CurrentStats() {
  oled.setFont(System5x7);
  oled.println("----- Overview -----");

  Serial.println(currentWeight);
  oled.print("Current weight: ");
  oled.println(currentWeight);
  oled.print("Minimum weight: ");
  oled.println(minWeightPlant);
  oled.print("Weight water: ");
  oled.println(weightWatering);

  oled.println("> Edit Settings");
}



void Menu2_Settings() {
  oled.clear();
  oled.println("--- Edit Settings ---");

  oled.println("Minimum weight plant:");

  if (menuStatus == 1) {
    oled.print(">");
    oled.print(" ");
    oled.print( minWeightPlant);
    oled.println(" ");
  } else {
    oled.println(minWeightPlant);
  }

  oled.println("Weight watering:");
  if (menuStatus == 2) {

    oled.print(">");
    oled.print(" ");
    oled.print(weightWatering );
    oled.println(" ");
  } else {
    oled.println(weightWatering);
  }


  if (menuStatus == 3) {
    oled.println("> Save");
  } else {
    oled.println("Save");
  }
}

void Menu2_SettingsLogic() {
  if (menuStatus == 1) {
    if (buttonState_mUp == HIGH && hasDoneAction_mUp == false) {
      minWeightPlant = minWeightPlant + 10;
      hasDoneAction_mUp = true;
    }

    if (buttonState_mDown == HIGH && minWeightPlant > 50 && hasDoneAction_mDown == false) {
      minWeightPlant = minWeightPlant - 10;
      hasDoneAction_mDown = true;
    }
  }


  if (menuStatus == 2) {
    if (buttonState_mUp == HIGH && hasDoneAction_mUp == false) {
      weightWatering = weightWatering + 10;
      hasDoneAction_mUp = true;
    }

    if (buttonState_mDown == HIGH && weightWatering > 50 && hasDoneAction_mDown == false) {
      weightWatering = weightWatering - 10;
      hasDoneAction_mDown = true;
    }
  }

  if (menuStatus == 3) {
    // nothing happens for now
  }



  if (buttonState_Click == HIGH && hasDoneAction_Click == false) {
    hasDoneAction_Click = true;
    menuStatus = menuStatus + 1;
  }

  EEPROM.update(EEPROM_WEIGHT_PLANT, minWeightPlant);
  EEPROM.update(EEPROM_WATER_WEIGHT, weightWatering);
}


void debouncingButtons() {
  int readingb_Click = digitalRead(PIN_BUTTON_CLICK);
  int readingb_mUp = digitalRead(PIN_BUTTON_MOVE_DOWN);
  int readingb_mDown = digitalRead(PIN_BUTTON_MOVE_UP);

  if (readingb_Click != lastButtonState_Click) {
    lastDebounceTime_Click = millis();
    Serial.println("set timer click");

  }

  if (readingb_mUp != lastButtonState_mUp) {
    lastDebounceTime_mUp = millis();
    // Serial.println("set timer up");
  }

  if (readingb_mDown != lastButtonState_mDown) {
    lastDebounceTime_mDown = millis();
    //Serial.println("set timer down");
  }

  if ((millis() - lastDebounceTime_Click) > debounceDelay) {
    // software debounce timer
    buttonState_Click = readingb_Click;

    if (buttonState_Click == LOW && hasDoneAction_Click == true) {
      // als button state = low en actie is uitgevoert
      hasDoneAction_Click = false;
    }

    if (buttonState_Click == HIGH) {
      Serial.println("Click pressed!");
    }

  }
  lastButtonState_Click = readingb_Click;

  if ((millis() - lastDebounceTime_mUp) > debounceDelay) {
    // software debounce timer
    buttonState_mUp = readingb_mUp;

    if (buttonState_mUp == LOW && hasDoneAction_mUp == true) {
      // als button state = low en actie is uitgevoert
      hasDoneAction_mUp = false;
    }

    if (buttonState_mUp == HIGH) {
      Serial.println("Move up pressed!");
      // do something if the button is pressed.
    }
  }

  lastButtonState_mUp = readingb_mUp;

  if ((millis() - lastDebounceTime_mDown) > debounceDelay) {
    // software debounce timer
    buttonState_mDown = readingb_mDown;

    if (buttonState_mDown == LOW && hasDoneAction_mDown == true) {
      // als button state = low en actie is uitgevoert
      hasDoneAction_mDown = false;
    }

    if (buttonState_mDown == HIGH) {
      Serial.println("Move down pressed!");
      // do something if the button is pressed.
    }
  }

  lastButtonState_mDown = readingb_mDown;
}
